@@ -16,9 +16,9 @@
 # Boston, MA 02111-1307, USA.
 
 import gtk
-import gobject
 import gettext
 import gconf
+import logging
 
 from sugar.graphics.toolbutton import ToolButton
 from sugar.graphics.toolbarbox import ToolbarButton
@@ -28,13 +28,14 @@ from sugar.graphics.toolbox import Toolbox
 from sugar.graphics.xocolor import XoColor
 from sugar.graphics.icon import Icon
 from sugar.bundle.activitybundle import ActivityBundle
+from sugar.graphics import style
 
 
 _ = lambda msg: gettext.dgettext('sugar-toolkit', msg)
 
 
 def _create_activity_icon(metadata):
-    if metadata.get('icon-color', ''):
+    if metadata is not None and metadata.get('icon-color'):
         color = XoColor(metadata['icon-color'])
     else:
         client = gconf.client_get_default()
@@ -162,10 +163,13 @@ class ShareButton(RadioMenuButton):
             self.neighborhood.handler_unblock(self._neighborhood_handle)
 
 
+# DEPRECATED
 class KeepButton(ToolButton):
 
     def __init__(self, activity, **kwargs):
         ToolButton.__init__(self, **kwargs)
+        logging.warning('KeepButton has been deprecated since Sugar 0.94'
+                        ' and should not be used in newly written code.')
         self.props.tooltip = _('Keep')
         self.props.accelerator = '<Ctrl>S'
 
@@ -186,48 +190,120 @@ class TitleEntry(gtk.ToolItem):
     def __init__(self, activity, **kwargs):
         gtk.ToolItem.__init__(self)
         self.set_expand(False)
-        self._update_title_sid = None
 
         self.entry = gtk.Entry(**kwargs)
         self.entry.set_size_request(int(gtk.gdk.screen_width() / 3), -1)
         self.entry.set_text(activity.metadata['title'])
-        self.entry.connect('changed', self.__title_changed_cb, activity)
+        self.entry.connect('focus-out-event', self.__title_changed_cb, activity)
         self.entry.show()
         self.add(self.entry)
 
         activity.metadata.connect('updated', self.__jobject_updated_cb)
+        activity.connect('_closing', self.__closing_cb)
 
     def modify_bg(self, state, color):
         gtk.ToolItem.modify_bg(self, state, color)
         self.entry.modify_bg(state, color)
 
     def __jobject_updated_cb(self, jobject):
+        if self.entry.flags() & gtk.HAS_FOCUS:
+            return
+        if self.entry.get_text() == jobject['title']:
+            return
         self.entry.set_text(jobject['title'])
 
-    def __title_changed_cb(self, entry, activity):
-        if self._update_title_sid is not None:
-            gobject.source_remove(self._update_title_sid)
-        self._update_title_sid = gobject.timeout_add_seconds(
-                1, self.__update_title_cb, activity)
+    def __closing_cb(self, activity):
+        self.save_title(activity)
+        return False
 
-    def __update_title_cb(self, activity):
+    def __title_changed_cb(self, editable, event, activity):
+        self.save_title(activity)
+        return False
+
+    def save_title(self, activity):
         title = self.entry.get_text()
+        if title == activity.metadata['title']:
+            return
 
         activity.metadata['title'] = title
         activity.metadata['title_set_by_user'] = '1'
         activity.save()
 
+        activity.set_title(title)
+
         shared_activity = activity.get_shared_activity()
         if shared_activity is not None:
             shared_activity.props.name = title
 
-        self._update_title_sid = None
+
+class DescriptionItem(gtk.ToolItem):
+
+    def __init__(self, activity, **kwargs):
+        gtk.ToolItem.__init__(self)
+
+        description_button = ToolButton('edit-description')
+        description_button.show()
+        description_button.set_tooltip(_('Description'))
+        self._palette = description_button.get_palette()
+
+        description_box = gtk.HBox()
+        sw = gtk.ScrolledWindow()
+        sw.set_size_request(int(gtk.gdk.screen_width() / 2),
+                            2 * style.GRID_CELL_SIZE)
+        sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self._text_view = gtk.TextView()
+        self._text_view.set_left_margin(style.DEFAULT_PADDING)
+        self._text_view.set_right_margin(style.DEFAULT_PADDING)
+        text_buffer = gtk.TextBuffer()
+        if 'description' in activity.metadata:
+            text_buffer.set_text(activity.metadata['description'])
+        self._text_view.set_buffer(text_buffer)
+        self._text_view.connect('focus-out-event',
+                               self.__description_changed_cb, activity)
+        sw.add(self._text_view)
+        description_box.pack_start(sw, False, True, 0)
+        self._palette.set_content(description_box)
+        description_box.show_all()
+
+        self.add(description_button)
+        description_button.connect('clicked',
+                                   self.__description_button_clicked_cb)
+
+        activity.metadata.connect('updated', self.__jobject_updated_cb)
+
+    def _get_text_from_buffer(self):
+        buf = self._text_view.get_buffer()
+        start_iter = buf.get_start_iter()
+        end_iter = buf.get_end_iter()
+        return buf.get_text(start_iter, end_iter, False)
+
+    def __jobject_updated_cb(self, jobject):
+        if self._text_view.has_focus():
+            return
+        if 'description' not in jobject:
+            return
+        if self._get_text_from_buffer() == jobject['description']:
+            return
+        buf = self._text_view.get_buffer()
+        buf.set_text(jobject['description'])
+
+    def __description_button_clicked_cb(self, button):
+        self._palette.popup(immediate=True, state=1)
+
+    def __description_changed_cb(self, widget, event, activity):
+        description = self._get_text_from_buffer()
+        if 'description' in activity.metadata and \
+                description == activity.metadata['description']:
+            return
+
+        activity.metadata['description'] = description
+        activity.save()
         return False
 
 
 class ActivityToolbar(gtk.Toolbar):
-    """The Activity toolbar with the Journal entry title, sharing,
-       Keep and Stop buttons
+    """The Activity toolbar with the Journal entry title, sharing
+       and Stop buttons
 
     All activities should have this toolbar. It is easiest to add it to your
     Activity by using the ActivityToolbox.
@@ -251,13 +327,17 @@ class ActivityToolbar(gtk.Toolbar):
             self.insert(separator, -1)
             separator.show()
 
+        if activity.metadata:
+            description_item = DescriptionItem(activity)
+            description_item.show()
+            self.insert(description_item, -1)
+
         self.share = ShareButton(activity)
         self.share.show()
         self.insert(self.share, -1)
 
+        # DEPRECATED
         self.keep = KeepButton(activity)
-        self.insert(self.keep, -1)
-        self.keep.show()
 
         self.stop = StopButton(activity)
         self.insert(self.stop, -1)
